@@ -14,18 +14,18 @@
 
 from netaddr import IPAddress
 
+from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
 from neutron_lib import context
+from neutron_lib import rpc as n_rpc
 from neutron_lib.services import base as service_base
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import importutils
 
-from neutron.common import rpc as n_rpc
-
 from neutron_dynamic_routing.api.rpc.agentnotifiers import bgp_dr_rpc_agent_api  # noqa
 from neutron_dynamic_routing.api.rpc.callbacks import resources as dr_resources
 from neutron_dynamic_routing.api.rpc.handlers import bgp_speaker_rpc as bs_rpc
@@ -95,6 +95,9 @@ class BgpPlugin(service_base.ServicePluginBase,
         registry.subscribe(self.router_gateway_callback,
                            resources.ROUTER_GATEWAY,
                            events.AFTER_DELETE)
+        registry.subscribe(self.port_callback,
+                           resources.PORT,
+                           events.AFTER_UPDATE)
 
     def get_bgp_speakers(self, context, filters=None, fields=None,
                          sorts=None, limit=None, marker=None,
@@ -294,14 +297,14 @@ class BgpPlugin(service_base.ServicePluginBase,
                                                     speaker.id,
                                                     rl)
 
-    def router_gateway_callback(self, resource, event, trigger, **kwargs):
+    def router_gateway_callback(self, resource, event, trigger, payload=None):
         if event == events.AFTER_CREATE:
-            self._handle_router_gateway_after_create(**kwargs)
+            self._handle_router_gateway_after_create(payload)
         if event == events.AFTER_DELETE:
-            gw_network = kwargs['network_id']
-            router_id = kwargs['router_id']
+            gw_network = payload.metadata.get('network_id')
+            router_id = payload.resource_id
             next_hops = self._next_hops_from_gateway_ips(
-                                                        kwargs['gateway_ips'])
+                payload.metadata.get('gateway_ips'))
             ctx = context.get_admin_context()
             speakers = self._bgp_speakers_for_gateway_network(ctx, gw_network)
             for speaker in speakers:
@@ -315,14 +318,15 @@ class BgpPlugin(service_base.ServicePluginBase,
                                                                      next_hop)
                 self._handle_router_interface_after_delete(gw_network, routes)
 
-    def _handle_router_gateway_after_create(self, **kwargs):
+    def _handle_router_gateway_after_create(self, payload):
         ctx = context.get_admin_context()
-        gw_network = kwargs['network_id']
-        router_id = kwargs['router_id']
+        gw_network = payload.metadata.get('network_id')
+        router_id = payload.resource_id
         with ctx.session.begin(subtransactions=True):
             speakers = self._bgp_speakers_for_gateway_network(ctx,
                                                               gw_network)
-            next_hops = self._next_hops_from_gateway_ips(kwargs['gw_ips'])
+            next_hops = self._next_hops_from_gateway_ips(
+                payload.metadata.get('gateway_ips'))
 
             for speaker in speakers:
                 if speaker.ip_version in next_hops:
@@ -344,6 +348,36 @@ class BgpPlugin(service_base.ServicePluginBase,
                 self.stop_route_advertisements(ctx, self._bgp_rpc,
                                                speaker.id, routes)
 
+    def port_callback(self, resource, event, trigger, **kwargs):
+        if event != events.AFTER_UPDATE:
+            return
+
+        original_port = kwargs['original_port']
+        updated_port = kwargs['port']
+        if not updated_port.get('fixed_ips'):
+            return
+
+        original_host = original_port.get(portbindings.HOST_ID)
+        updated_host = updated_port.get(portbindings.HOST_ID)
+        device_owner = updated_port.get('device_owner')
+
+        # if host in the port binding has changed, update next-hops
+        if original_host != updated_host and bool('compute:' in device_owner):
+            ctx = context.get_admin_context()
+            with ctx.session.begin(subtransactions=True):
+                ext_nets = self.get_external_networks_for_port(ctx,
+                                                               updated_port)
+                for ext_net in ext_nets:
+                    bgp_speakers = (
+                        self._get_bgp_speaker_ids_by_binding_network(
+                            ctx, ext_nets))
+
+                    # Refresh any affected BGP speakers
+                    for bgp_speaker in bgp_speakers:
+                        routes = self.get_advertised_routes(ctx, bgp_speaker)
+                        self.start_route_advertisements(ctx, self._bgp_rpc,
+                                                        bgp_speaker, routes)
+
     def _next_hops_from_gateway_ips(self, gw_ips):
         if gw_ips:
             return {IPAddress(ip).version: ip for ip in gw_ips}
